************************************************************
* Project:	ultraDev
* Module:	UDDebugScreen
* Code:		ultra
* Created:	22.12.2019
* Notes:	Service routines for debug screen printing
* Modified:
* Changes:
*
* (c) 2019 cream
************************************************************

UDDbgScrClear:
; clears the screen and sets all colors to white
; ->d0 clear value in ascii

	lea		$fb0000,a0

	and		#$ff,d0
	sub		#$20,d0
	move	UDRegWriteToScreenPos,d1
	move	(a0),d1			;set position to 0

	move	UDRegWriteToScreen,d1
	or		#$700,d0
	add		d0,d0
	move	#(UDScrCharsX*UDScrCharsY)-1,d1
.UDDbgScrClearLoop:
	move	(a0,d0),d2		;write char
	dbf		d1, .UDDbgScrClearLoop
	rts

****************************************************************************
UDDbgScrSetPosition:
; sets the printing position
; ->d0 x
; ->d1 y
	movem.l	d0-d1/a0,-(a7)
	mulu	#UDScrCharsX,d1
	add		d0,d1
	add		d1,d1

	move	UDRegWriteToScreenPos,d0
	lea		$fb0000,a0
	move	(a0,d1),d0
	movem.l	(a7)+,d0-d1/a0
	rts

****************************************************************************
UDDbgScrPrint:
; prints a 0 DbgScrinated string in the given color
; handles returns with 10. if the printed text reachs the screen bottom 
; it will be printed outside.
; 01,<color> will set the color of the text 
; ->a0 text
; ->d0 x
; ->d1 y
; ->d2 color
	
	bsr		UDDbgScrSetPosition
	
	and		#7,d2
	lsl		#8,d2
	move	UDRegWriteToScreen,d3
	lea		$fb0000,a6

.UDDbgScrPrintLoop:
	move.b	(a0)+,d2
	beq		.UDDbgScrPrintEnd

	cmp.b	#10,d2			; handle returns
	bne.s	.UDDbgScrPrintNoReturn
	addq	#1,d1
	bsr		UDDbgScrSetPosition

	move	UDRegWriteToScreen,d3
	bra.s	.UDDbgScrPrintLoop
.UDDbgScrPrintNoReturn:

	cmp.b	#1,d2			;handle colors
	bne.s	.UDDbgScrNoCol
	moveq	#0,d2
	move.b	(a0)+,d2
	lsl		#8,d2
	bra.s	.UDDbgScrPrintLoop

.UDDbgScrNoCol:
	cmp.b	#'%',d2			;handle hex printing
	bne.s	.UDDbgScrNoPrintHex
	move.b	(a0)+,d2

	cmp.b	#'b',d2
	bne.s	.UDDbgScrNoPrintHexByte
	move.b	(a1),d7
	bsr		UDDbgScrPrintByte

	bra.s	.UDDbgScrScrollVarArgs
.UDDbgScrNoPrintHexByte:

	cmp.b	#'w',d2
	bne.s	.UDDbgScrNoPrintHexWord
	move.b	(a1)+,d7
	bsr		UDDbgScrPrintByte
	move.b	(a1)+,d7
	bsr		UDDbgScrPrintByte

	bra.s	.UDDbgScrScrollVarArgs

.UDDbgScrNoPrintHexWord:
	cmp.b	#'l',d2
	bne.s	.UDDbgScrNoPrintHexLong
	move.b	(a1)+,d7
	bsr		UDDbgScrPrintByte
	move.b	(a1)+,d7
	bsr		UDDbgScrPrintByte
	move.b	(a1)+,d7
	bsr		UDDbgScrPrintByte
	move.b	(a1)+,d7
	bsr		UDDbgScrPrintByte
	bra.s	.UDDbgScrScrollVarArgs

.UDDbgScrNoPrintHexLong:
	bra.s	.UDDbgScrNoPrintHex

.UDDbgScrScrollVarArgs:
	move.l	a2,a1
	move.l	a3,a2
	move.l	a4,a3
	move.l	a5,a4
	bra		.UDDbgScrPrintLoop
.UDDbgScrNoPrintHex:


	sub.b	#$20,d2
	move	d2,d3
	add		d3,d3
	move	(a6,d3),d3		;write char to screen

	bra		.UDDbgScrPrintLoop

.UDDbgScrPrintEnd:
	rts

****************************************************************************
UDDbgScrPrintByte:
;intern use only !
	movem.l	a0/d2/d6/d7,-(a7)
	lea		UDDbgScrHex,a0
	and		#$ff,d7

	move	d7,d6
	and		#$f0,d6
	lsr		#4,d6
	move.b	(a0,d6),d2
	sub.b	#$20,d2
	move	d2,d6
	add		d6,d6
	move	(a6,d6),d3		;write char to screen

	and		#$f,d7
	move.b	(a0,d7),d2
	sub.b	#$20,d2
	move	d2,d6
	add		d6,d6
	move	(a6,d6),d3		;write char to screen
	movem.l	(a7)+,a0/d2/d6/d7
	rts

UDDbgScrHex: dc.b "0123456789abcdef"
